////////////////////
// setup GUI methods
////////////////////

private void setupControl() {
  noStroke();
  controlP5 = new ControlP5(this);
  controlP5.setAutoDraw(false);
  setParameters();
  createTabs();
  createCNCTab();
  createRobotTab();
  createAboutTab();
}

private void drawGUI() {
  pCamera.beginHUD();
  fill(0, 0, 0, 150);
  stroke(0, 0, 0);
  strokeWeight(1);
  rect(0, 0, GUIWidth, GUIHeight);
  controlP5.draw();
  pCamera.endHUD();
}

private void setParameters() {
  GUIWidth = width / 5;
  GUIHeight = height;
}


////////////////////////
// creating tabs methods
////////////////////////

private void createTabs() {
  controlP5.tab("default").hide();
  controlP5.addTab("CNC").setHeight(TABSHeight);
  controlP5.addTab("ROBOT").setHeight(TABSHeight);
  controlP5.addTab("ABOUT").setHeight(TABSHeight);
}

private void createCNCTab() {
  // add One-Layer group under CNC tab
  createCNCOneLayer();
  // add Multi_Layer group under CNC tab
  createCNCMltLayer();
}

private void createCNCOneLayer() {
  // create and modify group properties
  CNCOneLayerGroup = controlP5.addGroup("cnc one layer",0, marginTABGroup, GUIWidth);
  CNCOneLayerGroup.setBarHeight(barHeight);
  CNCOneLayerGroup.close();
  CNCOneLayerGroup.setBackgroundColor(color(0,200));
  CNCOneLayerHeight = (int) (3.0/5.0 * (height - marginTABGroup - barHeight - 2));
  CNCOneLayerGroup.setBackgroundHeight(CNCOneLayerHeight);
  CNCOneLayerGroup.activateEvent(true);
  CNCOneLayerGroup.moveTo("CNC");
  int toggleHeight = CNCOneLayerHeight / 16;
  // add display toggle
  CNCOneLayerDisp = controlP5.addToggle("display", false, GUIWidth/20, 1 * GUIWidth/20, GUIWidth - 2*GUIWidth/20, toggleHeight);
  CNCOneLayerDisp.setGroup(CNCOneLayerGroup);
  // add tessellation toggle
  CNCOneLayerTess = controlP5.addToggle("tessellation", false, GUIWidth/20, 1 * (toggleHeight + 5) + 2 * GUIWidth/20, GUIWidth - 2*GUIWidth/20, toggleHeight);
  CNCOneLayerTess.setGroup(CNCOneLayerGroup);
  // add additive toggle
  CNCOneLayerAddt = controlP5.addToggle("additive", false, GUIWidth/20, 2 * (toggleHeight + 5) + 3 * GUIWidth/20, GUIWidth - 2*GUIWidth/20, toggleHeight);
  CNCOneLayerAddt.setGroup(CNCOneLayerGroup);
  // add send to grasshopper toggle
  CNCOneLayerSend = controlP5.addToggle("send to grasshopper", false, GUIWidth/20, 3 * (toggleHeight + 5) + 4 * GUIWidth/20, GUIWidth - 2*GUIWidth/20, toggleHeight);
  CNCOneLayerSend.setGroup(CNCOneLayerGroup);
  int textfieldHeight = CNCOneLayerHeight / 10;
  // add cnc geometry lebgth number box
  CNCOneLayerTFLength = controlP5.addTextfield("base length", GUIWidth/20, 4 * (toggleHeight + 5) + 5 + 5 * GUIWidth/20, (GUIWidth - 3*GUIWidth/20) / 2, textfieldHeight);
  CNCOneLayerTFLength.setGroup(CNCOneLayerGroup);
  // add cnc geometry width number box
  CNCOneLayerTFWidth = controlP5.addTextfield("base width", GUIWidth/20, 4 * (toggleHeight + 5) + (textfieldHeight + 15) + 6 * GUIWidth/20, (GUIWidth - 3*GUIWidth/20) / 2, textfieldHeight);
  CNCOneLayerTFWidth.setGroup(CNCOneLayerGroup);
  // add cnc geometry height number box
  CNCOneLayerTFHeight = controlP5.addTextfield("base height", GUIWidth/20, 4 * (toggleHeight + 5) + (2 * textfieldHeight + 25) + 7 * GUIWidth/20, (GUIWidth - 3*GUIWidth/20) / 2, textfieldHeight);
  CNCOneLayerTFHeight.setGroup(CNCOneLayerGroup);
  // add cnc tessellation U number box
  CNCOneLayerTFGridU = controlP5.addTextfield("cnc tessellation U", 2 * GUIWidth/20 + (GUIWidth - 3*GUIWidth/20) / 2, 4 * (toggleHeight + 5) + 5 + 5 * GUIWidth/20, (GUIWidth - 3*GUIWidth/20) / 2, textfieldHeight);
  CNCOneLayerTFGridU.setGroup(CNCOneLayerGroup);
  CNCOneLayerTFGridU.hide();
  // add cnc tessellation V number box
  CNCOneLayerTFGridV = controlP5.addTextfield("cnc tessellation V", 2 * GUIWidth/20 + (GUIWidth - 3*GUIWidth/20) / 2, 4 * (toggleHeight + 5) + (textfieldHeight + 15) + 6 * GUIWidth/20, (GUIWidth - 3*GUIWidth/20) / 2, textfieldHeight);
  CNCOneLayerTFGridV.setGroup(CNCOneLayerGroup);
  CNCOneLayerTFGridV.hide();
}

private void createCNCMltLayer() {
  // create and modify group properties
  CNCMltLayerGroup = controlP5.addGroup("cnc multi layer",0, marginTABGroup + barHeight + 1, GUIWidth);
  CNCMltLayerGroup.setBarHeight(barHeight);
  CNCMltLayerGroup.close();
  CNCMltLayerGroup.setBackgroundColor(color(0,200));
  CNCMltLayerHeight = (int) (2.0/5.0 * (height - marginTABGroup - barHeight - 2));
  CNCMltLayerGroup.setBackgroundHeight(CNCMltLayerHeight);
  CNCMltLayerGroup.activateEvent(true);
  CNCMltLayerGroup.moveTo("CNC");
  int toggleHeight = CNCOneLayerHeight / 16;
  // add display toggle
  CNCMltLayerDisp = controlP5.addToggle("display", false, GUIWidth/20, 1 * GUIWidth/20, GUIWidth - 2*GUIWidth/20, toggleHeight);
  CNCMltLayerDisp.setGroup(CNCMltLayerGroup);
  // add autonumbring toggle
  CNCMltLayerANum = controlP5.addToggle("auto numbring", false, GUIWidth/20, 1 * (toggleHeight + 5) + 2 * GUIWidth/20, GUIWidth - 2*GUIWidth/20, toggleHeight);
  CNCMltLayerANum.setGroup(CNCMltLayerGroup);
  // add additive toggle
  CNCMltLayerAddt = controlP5.addToggle("additive", false, GUIWidth/20, 2 * (toggleHeight + 5) + 3 * GUIWidth/20, GUIWidth - 2*GUIWidth/20, toggleHeight);
  CNCMltLayerAddt.setGroup(CNCMltLayerGroup);
  // add send to grasshopper toggle
  CNCMltLayerSend = controlP5.addToggle("send to grasshopper", false, GUIWidth/20, 3 * (toggleHeight + 5) + 4 * GUIWidth/20, GUIWidth - 2*GUIWidth/20, toggleHeight);
  CNCMltLayerSend.setGroup(CNCMltLayerGroup);
  int textfieldHeight = CNCOneLayerHeight / 10;
  // add molds minimum thickness lebgth number box
  CNCMltLayerMinTHK = controlP5.addTextfield("mold min thickness", GUIWidth/20, 4 * (toggleHeight + 5) + 5 + 5 * GUIWidth/20, (GUIWidth - 3*GUIWidth/20) / 2, textfieldHeight);
  CNCMltLayerMinTHK.setGroup(CNCMltLayerGroup);
  // add molds maximum thickness tessellation U number box
  CNCMltLayerMaxTHK = controlP5.addTextfield("mold max thickness", 2 * GUIWidth/20 + (GUIWidth - 3*GUIWidth/20) / 2, 4 * (toggleHeight + 5) + 5 + 5 * GUIWidth/20, (GUIWidth - 3*GUIWidth/20) / 2, textfieldHeight);
  CNCMltLayerMaxTHK.setGroup(CNCMltLayerGroup);
}

private void createRobotTab() {
  // add One-Layer group under Robot tab
  createRBTOneLayer();
  // add Multi_Layer group under Robot tab
  createRBTMltLayer();
}

private void createRBTOneLayer() {
  // create and modify group properties
  RBTOneLayerGroup = controlP5.addGroup("robot one layer",0, marginTABGroup, GUIWidth);
  RBTOneLayerGroup.setBarHeight(barHeight);
  RBTOneLayerGroup.close();
  RBTOneLayerGroup.setBackgroundColor(color(0,200));
  RBTOneLayerHeight = (int) (3.0/5.0 * (height - marginTABGroup - barHeight - 2));
  RBTOneLayerGroup.setBackgroundHeight(RBTOneLayerHeight);
  RBTOneLayerGroup.activateEvent(true);
  RBTOneLayerGroup.moveTo("ROBOT");
  int toggleHeight = CNCOneLayerHeight / 16;
  // add display toggle
  RBTOneLayerDisp = controlP5.addToggle("display", false, GUIWidth/20, 1 * GUIWidth/20, GUIWidth - 2*GUIWidth/20, toggleHeight);
  RBTOneLayerDisp.setGroup(RBTOneLayerGroup);
  // add tessellation toggle
  RBTOneLayerTess = controlP5.addToggle("tessellation", false, GUIWidth/20, 1 * (toggleHeight + 5) + 2 * GUIWidth/20, GUIWidth - 2*GUIWidth/20, toggleHeight);
  RBTOneLayerTess.setGroup(RBTOneLayerGroup);
  // add additive toggle
  RBTOneLayerAddt = controlP5.addToggle("additive", false, GUIWidth/20, 2 * (toggleHeight + 5) + 3 * GUIWidth/20, GUIWidth - 2*GUIWidth/20, toggleHeight);
  RBTOneLayerAddt.setGroup(RBTOneLayerGroup);
  // add send to grasshopper toggle
  RBTOneLayerSend = controlP5.addToggle("send to grasshopper", false, GUIWidth/20, 3 * (toggleHeight + 5) + 4 * GUIWidth/20, GUIWidth - 2*GUIWidth/20, toggleHeight);
  RBTOneLayerSend.setGroup(RBTOneLayerGroup);
  int textfieldHeight = CNCOneLayerHeight / 10;
  // add robot geometry length number box
  RBTOneLayerTFLength = controlP5.addTextfield("base length", GUIWidth/20, 4 * (toggleHeight + 5) + 5 + 5 * GUIWidth/20, (GUIWidth - 3*GUIWidth/20) / 2, textfieldHeight);
  RBTOneLayerTFLength.setGroup(RBTOneLayerGroup);
  // add robot geometry width number box
  RBTOneLayerTFWidth = controlP5.addTextfield("base width", GUIWidth/20, 4 * (toggleHeight + 5) + (textfieldHeight + 15) + 6 * GUIWidth/20, (GUIWidth - 3*GUIWidth/20) / 2, textfieldHeight);
  RBTOneLayerTFWidth.setGroup(RBTOneLayerGroup);
  // add robot geometry height number box
  RBTOneLayerTFHeight = controlP5.addTextfield("base height", GUIWidth/20, 4 * (toggleHeight + 5) + (2 * textfieldHeight + 25) + 7 * GUIWidth/20, (GUIWidth - 3*GUIWidth/20) / 2, textfieldHeight);
  RBTOneLayerTFHeight.setGroup(RBTOneLayerGroup);
  // add robot tessellation U number box
  RBTOneLayerTFGridU = controlP5.addTextfield("grid U", 2 * GUIWidth/20 + (GUIWidth - 3*GUIWidth/20) / 2, 4 * (toggleHeight + 5) + 5 + 5 * GUIWidth/20, (GUIWidth - 3*GUIWidth/20) / 2, textfieldHeight);
  RBTOneLayerTFGridU.setGroup(RBTOneLayerGroup);
  RBTOneLayerTFGridU.hide();
  // add robot tessellation V number box
  RBTOneLayerTFGridV = controlP5.addTextfield("grid V", 2 * GUIWidth/20 + (GUIWidth - 3*GUIWidth/20) / 2, 4 * (toggleHeight + 5) + (textfieldHeight + 15) + 6 * GUIWidth/20, (GUIWidth - 3*GUIWidth/20) / 2, textfieldHeight);
  RBTOneLayerTFGridV.setGroup(RBTOneLayerGroup);
  RBTOneLayerTFGridV.hide();
  // add robot tessellation V number box
  RBTOneLayerTFGridW = controlP5.addTextfield("grid W", 2 * GUIWidth/20 + (GUIWidth - 3*GUIWidth/20) / 2, 4 * (toggleHeight + 5) + (2 * textfieldHeight + 25) + 7 * GUIWidth/20, (GUIWidth - 3*GUIWidth/20) / 2, textfieldHeight);
  RBTOneLayerTFGridW.setGroup(RBTOneLayerGroup);
  RBTOneLayerTFGridW.hide();
}

private void createRBTMltLayer() {
  // create and modify group properties
  RBTMltLayerGroup = controlP5.addGroup("robot multi layer",0, marginTABGroup + barHeight + 1, GUIWidth);
  RBTMltLayerGroup.setBarHeight(barHeight);
  RBTMltLayerGroup.close();
  RBTMltLayerGroup.setBackgroundColor(color(0,200));
  RBTMltLayerHeight = (int) (2.0/5.0 * (height - marginTABGroup - barHeight - 2));
  RBTMltLayerGroup.setBackgroundHeight(CNCMltLayerHeight);
  RBTMltLayerGroup.activateEvent(true);
  RBTMltLayerGroup.moveTo("ROBOT");
  int toggleHeight = CNCOneLayerHeight / 16;
  // add display toggle
  RBTMltLayerDisp = controlP5.addToggle("display", false, GUIWidth/20, 1 * GUIWidth/20, GUIWidth - 2*GUIWidth/20, toggleHeight);
  RBTMltLayerDisp.setGroup(RBTMltLayerGroup);
  // add autonumbring toggle
  RBTMltLayerANum = controlP5.addToggle("auto numbring", false, GUIWidth/20, 1 * (toggleHeight + 5) + 2 * GUIWidth/20, GUIWidth - 2*GUIWidth/20, toggleHeight);
  RBTMltLayerANum.setGroup(RBTMltLayerGroup);
  // add additive toggle
  RBTMltLayerAddt = controlP5.addToggle("additive", false, GUIWidth/20, 2 * (toggleHeight + 5) + 3 * GUIWidth/20, GUIWidth - 2*GUIWidth/20, toggleHeight);
  RBTMltLayerAddt.setGroup(RBTMltLayerGroup);
  // add send to grasshopper toggle
  RBTMltLayerSend = controlP5.addToggle("send to grasshopper", false, GUIWidth/20, 3 * (toggleHeight + 5) + 4 * GUIWidth/20, GUIWidth - 2*GUIWidth/20, toggleHeight);
  RBTMltLayerSend.setGroup(RBTMltLayerGroup);
  int textfieldHeight = CNCOneLayerHeight / 10;
  // add molds minimum thickness lebgth number box
  RBTMltLayerMinTHK = controlP5.addTextfield("mold min thickness", GUIWidth/20, 4 * (toggleHeight + 5) + 5 + 5 * GUIWidth/20, (GUIWidth - 3*GUIWidth/20) / 2, textfieldHeight);
  RBTMltLayerMinTHK.setGroup(RBTMltLayerGroup);
  // add molds maximum thickness tessellation U number box
  RBTMltLayerMaxTHK = controlP5.addTextfield("mold max thickness", 2 * GUIWidth/20 + (GUIWidth - 3*GUIWidth/20) / 2, 4 * (toggleHeight + 5) + 5 + 5 * GUIWidth/20, (GUIWidth - 3*GUIWidth/20) / 2, textfieldHeight);
  RBTMltLayerMaxTHK.setGroup(RBTMltLayerGroup);
}


private void createAboutTab() {
}


///////////////////////////
// handeling events methods
///////////////////////////

// this method controls event if a GUI event happens
private void controlEvent(ControlEvent theEvent) {
  if(theEvent.isGroup()) {
    groupEvent(theEvent);
  } 
  if(theEvent.isController()) {
    controllerEvent(theEvent);
  }
}

// this method controls event if a Group event happens
private void groupEvent(ControlEvent theEvent) {
  // change group positions when upper group closes or opens
  if (theEvent.name().equals("cnc one layer")) {
    if (CNCOneLayerGroup.isOpen()) {
      CNCMltLayerGroup.setPosition(0, marginTABGroup + CNCOneLayerHeight + 1 + barHeight + 1);
    } else {
      CNCMltLayerGroup.setPosition(0, marginTABGroup + barHeight + 1);
    }
  } else if(theEvent.name().equals("robot one layer")) {
    if (RBTOneLayerGroup.isOpen()) {
      RBTMltLayerGroup.setPosition(0, marginTABGroup + RBTOneLayerHeight + 1 + barHeight + 1);
    } else {
      RBTMltLayerGroup.setPosition(0, marginTABGroup + barHeight + 1);
    }
  }
}

// this method controls event if a Controller event happens
private void controllerEvent(ControlEvent theEvent) {
  if (theEvent.controller() == CNCOneLayerDisp) {
    readCNCOneLayerTextFields();
    fillCOLData();
    CNCOneLayerDispEvent();
  } else if (theEvent.controller() == CNCOneLayerTess) {
    readCNCOneLayerTextFields();
    CNCOneLayerTessEvent();
    fillCOLData();
  } else if (theEvent.controller() == CNCOneLayerAddt) {
    readCNCOneLayerTextFields();
    //fillCOLData();
  } else if (theEvent.controller() == CNCOneLayerSend) {
    readCNCOneLayerTextFields();
    //fillCOLData();
    CNCOneLayerSendEvent();
  } else if (theEvent.controller() == CNCMltLayerDisp) {
    readCNCMltLayerTextFields();
    CNCMltLayerDispEvent();
    fillCMLData();
  } else if (theEvent.controller() == CNCMltLayerANum) {
    readCNCMltLayerTextFields();
  } else if (theEvent.controller() == CNCMltLayerAddt) {
    readCNCMltLayerTextFields();
    fillCMLData();
  } else if (theEvent.controller() == CNCMltLayerSend) {
    readCNCMltLayerTextFields();
    CNCMltLayerSendEvent();
  } else if (theEvent.controller() == RBTOneLayerDisp) {
    readRBTOneLayerTextFields();
    RBTOneLayerDispEvent();
    fillROLData();
  } else if (theEvent.controller() == RBTOneLayerTess) {
    readRBTOneLayerTextFields();
    RBTOneLayerTessEvent();
  } else if (theEvent.controller() == RBTOneLayerAddt) {
    readRBTOneLayerTextFields();
    fillROLData();
  } else if (theEvent.controller() == RBTOneLayerSend) {
    readRBTOneLayerTextFields();
    RBTOneLayerSendEvent();
  } else if (theEvent.controller() == RBTMltLayerDisp) {
    readRBTMltLayerTextFields();
    RBTMltLayerDispEvent();
    fillRMLData();
  } else if (theEvent.controller() == RBTMltLayerANum) {
    readRBTMltLayerTextFields();
  } else if (theEvent.controller() == RBTMltLayerAddt) {
    readRBTMltLayerTextFields();
    fillRMLData();
  } else if (theEvent.controller() == RBTMltLayerSend) {
    readRBTMltLayerTextFields();
    RBTMltLayerSendEvent();
  }
}

// this method controls event if a CNCOneLayerTess event happens
private void CNCOneLayerTessEvent() {
  // show grid u and v textfields
  if (CNCOneLayerTess.getState() == true) {
    CNCOneLayerTFGridU.show();
    CNCOneLayerTFGridV.show();
  // hide grid u and v textfields
  } else {
    CNCOneLayerTFGridU.hide();
    CNCOneLayerTFGridV.hide();
  }
}

// this method controls event if a RBTOneLayerTess event happens
void RBTOneLayerTessEvent() {
  // show grid u, v and w textfields
  if (RBTOneLayerTess.getState() == true) {
    RBTOneLayerTFGridU.show();
    RBTOneLayerTFGridV.show();
    RBTOneLayerTFGridW.show();
  // hide grid u, v and w textfields
  } else {
    RBTOneLayerTFGridU.hide();
    RBTOneLayerTFGridV.hide();
    RBTOneLayerTFGridW.hide();
  }
}

// this method controls event if a CNCOneLayerDisp event happens
private void CNCOneLayerDispEvent() {
  // turn off other display toggles
  if (CNCOneLayerDisp.getState() == true) {
    CNCMltLayerDisp.setState(false);
    RBTOneLayerDisp.setState(false);
    RBTMltLayerDisp.setState(false);
  }
}

// this method controls event if a CNCMltLayerDisp event happens
private void CNCMltLayerDispEvent() {
  // turn off other display toggles
  if (CNCMltLayerDisp.getState() == true) {
    CNCOneLayerDisp.setState(false);
    RBTOneLayerDisp.setState(false);
    RBTMltLayerDisp.setState(false);
  }
}

// this method controls event if a RBTOneLayerDisp event happens
private void RBTOneLayerDispEvent() {
  // turn off other display toggles
  if (RBTOneLayerDisp.getState() == true) {
    CNCOneLayerDisp.setState(false);
    CNCMltLayerDisp.setState(false);
    RBTMltLayerDisp.setState(false);
  }
}

// this method controls event if a RBTMltLayerDisp event happens
private void RBTMltLayerDispEvent() {
  // turn off other display toggles
  if (RBTMltLayerDisp.getState() == true) {
    CNCOneLayerDisp.setState(false);
    CNCMltLayerDisp.setState(false);
    RBTOneLayerDisp.setState(false);
  }
}

// this method controls event if a CNCOneLayerSend event happens
private void CNCOneLayerSendEvent() {
  // turn off other display toggles
  if (CNCOneLayerSend.getState() == true) {
    // if base material dimensions are invalid turn off sending
    if (CNCOneLayerTFLengthValue == 0 || 
        CNCOneLayerTFWidthValue == 0 || 
        CNCOneLayerTFHeightValue == 0) {
      println("Invalid base dimensions.");
    } else {
      writeCOLData();
    }
    CNCOneLayerSend.setState(false);
  }
}

// this method controls event if a CNCMltLayerSend event happens
private void CNCMltLayerSendEvent() {
  // turn off other display toggles
  if (CNCMltLayerSend.getState() == true) {
    // if mold dimensions are invalid turn off sending
    if (CNCMltLayerMinTHKValue == 0 || 
        CNCMltLayerMaxTHKValue == 0) {
      println("Invalid MIN or MAX thickness.");
    } else {
      writeCMLData();
    }
    CNCMltLayerSend.setState(false);
  }
}

// this method controls event if a RBTOneLayerSend event happens
private void RBTOneLayerSendEvent() {
  // turn off other display toggles
  if (RBTOneLayerSend.getState() == true) {
    // if base material dimensions are invalid turn off sending
    if (RBTOneLayerTFLengthValue == 0 || 
        RBTOneLayerTFWidthValue == 0 || 
        RBTOneLayerTFHeightValue == 0) {
      println("Invalid base dimensions.");
    } else {
      writeROLData();
    }
    RBTOneLayerSend.setState(false);
  }
}

// this method controls event if a RBTMltLayerSend event happens
private void RBTMltLayerSendEvent() {
  // turn off other display toggles
  if (RBTMltLayerSend.getState() == true) {
    // if mold dimensions are invalid turn off sending
    if (RBTMltLayerMinTHKValue == 0 || 
        RBTMltLayerMaxTHKValue == 0) {
      println("Invalid MIN or MAX thickness.");
    } else {
      writeRMLData();
    }
    RBTMltLayerSend.setState(false);
  }
}


////////////////////////////////////////////////
// reading data from text fields related methods
////////////////////////////////////////////////

// this method read cnc one layer text fields string and parse them to values
private void readCNCOneLayerTextFields() {
  
  // read cnc one layer textfields
  try {
    if (!CNCOneLayerTFLength.getText().isEmpty()) {
      CNCOneLayerTFLengthValue = Float.parseFloat(CNCOneLayerTFLength.getText());
    } else {
      CNCOneLayerTFLengthValue = 0;
    }
  } catch (NumberFormatException e) {
    CNCOneLayerTFLengthValue = 0;
    println("CNC one layer length value is not valid");
  }
  
  try {
    if (!CNCOneLayerTFWidth.getText().isEmpty()) {
      CNCOneLayerTFWidthValue = Float.parseFloat(CNCOneLayerTFWidth.getText());
    } else {
      CNCOneLayerTFWidthValue = 0;
    }
  } catch (NumberFormatException e) {
    CNCOneLayerTFWidthValue = 0;
    println("CNC one layer width value is not valid");
  }
  
  try {
    if (!CNCOneLayerTFHeight.getText().isEmpty()) {
      CNCOneLayerTFHeightValue = Float.parseFloat(CNCOneLayerTFHeight.getText());
    } else {
      CNCOneLayerTFHeightValue = 0;
    }
  } catch (NumberFormatException e) {
    CNCOneLayerTFHeightValue = 0;
    println("CNC one layer height value is not valid");
  }
  
  try {
    if (!CNCOneLayerTFGridU.getText().isEmpty()) {
      CNCOneLayerTFGridUValue = Integer.parseInt(CNCOneLayerTFGridU.getText());
    } else {
      CNCOneLayerTFGridUValue = 1;
    }
  } catch (NumberFormatException e) {
    CNCOneLayerTFGridUValue = 1;
    println("CNC one layer gridu value is not valid");
  }
  
  try {
    if (!CNCOneLayerTFGridV.getText().isEmpty()) {
      CNCOneLayerTFGridVValue = Integer.parseInt(CNCOneLayerTFGridV.getText());
    } else {
      CNCOneLayerTFGridVValue = 1;
    }
  } catch (NumberFormatException e) {
    CNCOneLayerTFGridVValue = 1;
    println("CNC one layer gridv value is not valid");
  }
}

// this method read cnc multiple layer text fields string and parse them to values
private void readCNCMltLayerTextFields() {
  
  // read cnc multiple layer textfields
  try {
    if (!CNCMltLayerMinTHK.getText().isEmpty()) {
     CNCMltLayerMinTHKValue = Float.parseFloat(CNCMltLayerMinTHK.getText());
    } else {
     CNCMltLayerMinTHKValue = 0;
    }
  } catch (NumberFormatException e) {
    CNCMltLayerMinTHKValue = 0;
    println("CNC multiple layer minimum thickness value is not valid");
  }
  
  try {
    if (!CNCMltLayerMaxTHK.getText().isEmpty()) {
     CNCMltLayerMaxTHKValue = Float.parseFloat(CNCMltLayerMaxTHK.getText());
    } else {
     CNCMltLayerMaxTHKValue = 0;
    }
  } catch (NumberFormatException e) {
    CNCMltLayerMaxTHKValue = 0;
    println("CNC multiple layer maximum thickness value is not valid");
  }
}

// this method read robot one layer text fields string and parse them to values
private void readRBTOneLayerTextFields() {
  
  // read robot one layer textfields
  try {
    if (!RBTOneLayerTFLength.getText().isEmpty()) {
      RBTOneLayerTFLengthValue = Float.parseFloat(RBTOneLayerTFLength.getText());
    } else {
      RBTOneLayerTFLengthValue = 0;
    }
  } catch (NumberFormatException e) {
    RBTOneLayerTFLengthValue = 0;
    println("Robot one layer length value is not valid");
  }
  
  try {
    if (!RBTOneLayerTFWidth.getText().isEmpty()) {
      RBTOneLayerTFWidthValue = Float.parseFloat(RBTOneLayerTFWidth.getText());
    } else {
      RBTOneLayerTFWidthValue = 0;
    }
  } catch (NumberFormatException e) {
    RBTOneLayerTFWidthValue = 0;
    println("Robot one layer width value is not valid");
  }
  
  try {
    if (!RBTOneLayerTFHeight.getText().isEmpty()) {
      RBTOneLayerTFHeightValue = Float.parseFloat(RBTOneLayerTFHeight.getText());
    } else {
      RBTOneLayerTFHeightValue = 0;
    }
  } catch (NumberFormatException e) {
    RBTOneLayerTFHeightValue = 0;
    println("Robot one layer height value is not valid");
  }
  
  try {
    if (!RBTOneLayerTFGridU.getText().isEmpty()) {
      RBTOneLayerTFGridUValue = Integer.parseInt(RBTOneLayerTFGridU.getText());
    } else {
      RBTOneLayerTFGridUValue = 1;
    }
  } catch (NumberFormatException e) {
    RBTOneLayerTFGridUValue = 1;
    println("Robot one layer gridu value is not valid");
  }
  
  try {
    if (!RBTOneLayerTFGridV.getText().isEmpty()) {
      RBTOneLayerTFGridVValue = Integer.parseInt(RBTOneLayerTFGridV.getText());
    } else {
      RBTOneLayerTFGridVValue = 1;
    }
  } catch (NumberFormatException e) {
    RBTOneLayerTFGridVValue = 1;
    println("Robot one layer gridv value is not valid");
  }
  
  try {
    if (!RBTOneLayerTFGridW.getText().isEmpty()) {
      RBTOneLayerTFGridWValue = Integer.parseInt(RBTOneLayerTFGridW.getText());
    } else {
      RBTOneLayerTFGridWValue = 1;
    }
  } catch (NumberFormatException e) {
    RBTOneLayerTFGridWValue = 1;
    println("Robot one layer gridw value is not valid");
  }
}

// this method read robot multiple layer text fields string and parse them to values
private void readRBTMltLayerTextFields() {
  
  // read robot multiple layer textfields
  try {
    if (!RBTMltLayerMinTHK.getText().isEmpty()) {
     RBTMltLayerMinTHKValue = Float.parseFloat(RBTMltLayerMinTHK.getText());
    } else {
     RBTMltLayerMinTHKValue = 0;
    }
  } catch (NumberFormatException e) {
    RBTMltLayerMinTHKValue = 0;
    println("Robot multiple layer minimum thickness value is not valid");
  }
  
  try {
    if (!RBTMltLayerMaxTHK.getText().isEmpty()) {
     RBTMltLayerMaxTHKValue = Float.parseFloat(RBTMltLayerMaxTHK.getText());
    } else {
     RBTMltLayerMaxTHKValue = 0;
    }
  } catch (NumberFormatException e) {
    RBTMltLayerMaxTHKValue = 0;
    println("Robot multiple layer maximum thickness value is not valid");
  }
}
