
/*
---------------------------
-- SUBTRACTIVE FABRICATOR--
---------------------------
*/

/* This is a code written by Mehrdad Ghods
 * as a project for course "60-412 : Special Topics: Computational Design and Interactive Art"
 * by Prof. Golan Levin

/* Description:
Note: this project is still under development and only CNC One Layer Tab functions are full functional.
      for newer versions of the code email mghods@andrew.cmu.edu
      
Note: for running this code you need to have PeasyCam and ControlP5 libraries installed

For running your class that implements subfaricator interface you should define it as a global
variable and also instantiate it in setup data method below

You can find more information about this project and other great projects here:
http://golancourses.net/2010spring/category/final-project/
*/
// -----------------------------------------------------------------------------

/////////
//Imports
/////////

import processing.opengl.*;
import hypermedia.net.*;
import peasy.*;
import controlP5.*;

//////////////
//Declerations
//////////////

PeasyCam pCamera;

// viewport values
float movementX;
float movementY;
float movementZ;
float rotationX;
float rotationY;
float rotationZ;
float zoom;

//// controlP5 related global objects
ControlP5 controlP5;
// CNC tab related GUI components
ControlGroup CNCOneLayerGroup;
Toggle CNCOneLayerDisp;
Toggle CNCOneLayerSend;
Toggle CNCOneLayerTess;
Toggle CNCOneLayerAddt;
Textfield CNCOneLayerTFLength;
Textfield CNCOneLayerTFWidth;
Textfield CNCOneLayerTFHeight;
Textfield CNCOneLayerTFGridU;
Textfield CNCOneLayerTFGridV;

ControlGroup CNCMltLayerGroup;
Toggle CNCMltLayerDisp;
Toggle CNCMltLayerANum;
Toggle CNCMltLayerAddt;
Toggle CNCMltLayerSend;
Slider CNCMltLayerPrecision;
Textfield CNCMltLayerMinTHK;
Textfield CNCMltLayerMaxTHK;
// ROBOT tab related GUI variables
int CNCOneLayerHeight;
int CNCMltLayerHeight;
// ROBOT tab related GUI components
ControlGroup RBTOneLayerGroup;
Toggle RBTOneLayerDisp;
Toggle RBTOneLayerSend;
Toggle RBTOneLayerTess;
Toggle RBTOneLayerAddt;
Textfield RBTOneLayerTFLength;
Textfield RBTOneLayerTFWidth;
Textfield RBTOneLayerTFHeight;
Textfield RBTOneLayerTFGridU;
Textfield RBTOneLayerTFGridV;
Textfield RBTOneLayerTFGridW;

ControlGroup RBTMltLayerGroup;
Toggle RBTMltLayerDisp;
Toggle RBTMltLayerANum;
Toggle RBTMltLayerAddt;
Toggle RBTMltLayerSend;
Slider RBTMltLayerPrecision;
Textfield RBTMltLayerMinTHK;
Textfield RBTMltLayerMaxTHK;

// ROBOT tab related GUI variables
int RBTOneLayerHeight;
int RBTMltLayerHeight;

//// controlP5 related global variables
int TABSHeight = 20;
int marginTABGroup = (int)(3.0 / 2.0 * TABSHeight) + 2;
int barHeight = 10;
int GUIWidth;
int GUIHeight;  

// CNC one layer textfields values
float CNCOneLayerTFLengthValue = 0;
float CNCOneLayerTFWidthValue = 0;
float CNCOneLayerTFHeightValue = 0;
int CNCOneLayerTFGridUValue = 1;
int CNCOneLayerTFGridVValue = 1;

// CNC multiple layer textfields values
float CNCMltLayerMinTHKValue = 0;
float CNCMltLayerMaxTHKValue = 0;

// Robot one layer textfields values
float RBTOneLayerTFLengthValue = 0;
float RBTOneLayerTFWidthValue = 0;
float RBTOneLayerTFHeightValue = 0;
int RBTOneLayerTFGridUValue = 1;
int RBTOneLayerTFGridVValue = 1;
int RBTOneLayerTFGridWValue = 1;

// Robot multiple layer textfields values
float RBTMltLayerMinTHKValue = 0;
float RBTMltLayerMaxTHKValue = 0;

//// data colors
// if data is valid color it green
color validDataColor = color(0, 255, 0, 50);
// if data is valid final drill cut is brown
color validDrillColor = color(101, 67, 33);
// if data is not valid color it red
color invalidDataColor = color (255, 0, 0);
// color of wood
color wood = color (152, 118, 84);

//// data lists to send to be displayed and sent to GH
HashSet<OneLayerDataToSend> COLDataToSend = new HashSet<OneLayerDataToSend>();
HashSet<OneLayerDataToSend> CMLDataToSend = new HashSet<OneLayerDataToSend>();
HashSet<OneLayerDataToSend> ROLDataToSend = new HashSet<OneLayerDataToSend>();
HashSet<OneLayerDataToSend> RMLDataToSend = new HashSet<OneLayerDataToSend>();

//// define UDP object and related variables
UDP udps;
// the remote IP address
String ip = "127.0.0.1";
// the destination port
int port = 6400;


//// define instances of your defined classes here ////
UserClass6 userClass;


///////////////
// main methods
///////////////

// this method setup main screen
void setup() {
  println("//// SETUP WARNINGS ////");
  setupControl();
  setupScreen();
  setupCamera();
  setupData();
  // instantiate udps object for recibveing on port 6000
  udps = new UDP( this, 6000 );
  println("//// SETUP COMPLETED ////");
  println();
  println("//// RUNNING WARNINGS ////");
}

// this method is main method of program
void draw() {
  updateScreen();
  setMouseAction();
  drawVectors();
  drawGUI();
  //sendDataToGH();
}


////////////////////////
// setup related methods
////////////////////////

// this method setup screen
void setupScreen() {
  size(int(screen.width/2), int(screen.height/2), OPENGL);
  frame.setLocation(screen.width/4, screen.height/4);
}

// this method setup camera
void setupCamera() {
  pCamera = new PeasyCam(this, 100);
}

// this method setup input data for fabrication
//// you must initialize your defined instances here ////
void setupData() {
  //userClass = new UserClass0(100, 100, 2);
  //userClass = new UserClass1(100, 100, 1);
  //userClass = new UserClass2(100, 100, 1);
  //userClass = new UserClass3(200, 200, 1);
  //userClass = new UserClass4();
  //userClass = new UserClass5(100, 100, 1);
  userClass = new UserClass6(100, 100, 1);
  fillData();
}


///////////////////////
// data filling methods
///////////////////////

// this method fill data from user defined object to data to send lists
void fillData() {
  fillCOLData();
  fillCMLData();
  fillROLData();
  fillRMLData();
}

// this method fill cnc one layer data to send arraylist
void fillCOLData() {
  // check if user defined class has valid CNC one layre data
  if (userClass.CNCOneLayer() == null || userClass.CNCOneLayer().isEmpty()) {
    println("COLData is not valid");
    return;
  }
  
  COLDataToSend = new HashSet<OneLayerDataToSend>();
  // copy user defined class CNC one layer data to CNC one layer data to send arraylist
  // read data from user defined objects and copy them to data to send arraylist one by one
  OneLayerDataToSend curDataOut;
  for(CNCOneLayerData curDataIn : userClass.CNCOneLayer()) {
    curDataOut = new OneLayerDataToSend(curDataIn.getPosition(), curDataIn.getMagnitude(), curDataIn.getDiameter(), 0, 0, true);
    curDataOut.setIsValid(isValidCNCOneLayer(curDataOut));
    COLDataToSend.add(curDataOut);
  }
}

// this method fill cnc multiple layer data to send arraylist
void fillCMLData() {
}

// this method fill robot one layer data to send arraylist
void fillROLData() {
}

// this method fill robot multiple layer data to send arraylist
void fillRMLData() {
}


//////////////////////////
// screen updating methods
//////////////////////////

// this method set mouse available actions based on mouse location on screen
void setMouseAction() {
  // if mouse is oveer menu disable camera controllers
  if (mouseX < GUIWidth && mouseX > 0) {
    pCamera.setMouseControlled(false);
    pCamera.setResetOnDoubleClick(false);
  } else  {
    pCamera.setMouseControlled(true);
    pCamera.setResetOnDoubleClick(true);
  }
}


// this method update what should be drawn
void updateScreen() {
  // make background gray
  background(10);
  drawGrids();
  drawArrows();
}

// this method draw coordinate arrows
void drawArrows() {
  // increase stroke weight base on window width
  strokeWeight(5);
  // red for X arrow
  stroke(255, 0, 0);
  // draw X arrow
  line(0, 0, 0, 10, 0, 0);
  // green for Y arrow
  stroke(0, 255, 0);
  // draw Y arrow
  line(0, 0, 0, 0, 10, 0);
  // blue for Z arrow
  stroke(0, 0, 255);
  // draw Z arrow
  line(0, 0, 0, 0, 0, 10);
}

// this method draw gridlines
void drawGrids() {
  strokeWeight(1);
  for(int i = -2000; i <= 2000; i += 50) {
    if (i == 0) stroke(0, 255, 0);
    else stroke(200);
    line(i, -2000, 0, i, 2000, 0);
    if (i == 0) stroke(255, 0, 0);
    else stroke(200);
    line(-2000, i, 0, 2000, i, 0);
  }
}


///////////////////////
// data drawing methods
///////////////////////

// this method handle drawing of input data as vectors
void drawVectors() {
  // display cnc one layer data
  if (CNCOneLayerDisp.getState()) {
    drawCOLData();
  }
  // display cnc multiple layer data
  if (RBTOneLayerDisp.getState()) {
    drawCMLData();
  }
  
  // display robot one layer data
  if (CNCMltLayerDisp.getState()) {
    drawROLData();
  }
  
  // display robot multiple layer data
  if (RBTMltLayerDisp.getState()) {
    drawRMLData();
  }
}

// this method display cnc one layer data
void drawCOLData() {
  // update COLData
  updateCOLData();
  
  // turn off cpu and memory consuming features
  noSmooth();
  strokeWeight(2);
  noFill();
  
  // check if there is data to visualize
  if (COLDataToSend != null && !COLDataToSend.isEmpty()) {
    // loop through all data in the COLData list
    for (OneLayerDataToSend curData : COLDataToSend) {
      // define start and end coordinates
      float x0 = curData.getStrPosition().x;
      float y0 = curData.getStrPosition().y;
      float z0 = curData.getStrPosition().z;
      float x1 = curData.getEndPosition().x;
      float y1 = curData.getEndPosition().y;
      float z1 = curData.getEndPosition().z;
      
      
      // visualize data based on if its an additive fabrication or a subtractive one
      if (CNCOneLayerAddt.getState() == true) {
        // if data is in boundary and valid but z0 is above top surface
        // put it on top surface of base geometry
        if(curData.getIsValid() && z0 > CNCOneLayerTFHeightValue) {
          z0 = CNCOneLayerTFHeightValue;
        }
        // draw remained part of base material as a line
        if(curData.getIsValid()) {
          stroke(wood);
        } else {
          stroke(invalidDataColor);
        }
        if (curData.getMagnitude() == 0) {continue;}
        line(x0, y0, z0, x1, y1, z1);
        pushMatrix();
        translate(0,0, z0);
        stroke(validDrillColor);
        rect(x0, y0, curData.getDiameter(), curData.getDiameter());
        popMatrix();        
        pushMatrix();
        translate(0,0, z1);
        stroke(validDrillColor);
        rect(x1, y1, curData.getDiameter(), curData.getDiameter());
        popMatrix();        
      } else {
        // draw remained part of base material as a line
        if((x1 >= 0 && x1 <= CNCOneLayerTFLengthValue) &&
           (y1 >= 0 && y1 <= CNCOneLayerTFWidthValue) &&
           (z1 >= 0 && z1 <= CNCOneLayerTFHeightValue)) {
          stroke(wood);
        } else {
          stroke(invalidDataColor);
        }
        if (curData.getMagnitude() == 0) {continue;}
        line(x0, y0, 0, x1, y1, z1);
        // draw end of drill path as a small rectangle
        pushMatrix();
        stroke(validDrillColor);
        translate(0,0, z1);
        rect(x1, y1, 0.3, 0.3);
        popMatrix();
      }
    }
  }
}

// this method display cnc multiple layer data
void drawCMLData() {
}

// this method display robot one layer data
void drawROLData() {
}

// this method display robot multiple layer data
void drawRMLData() {
}


////////////////////////
// data updating methods
////////////////////////

// this method update COLData set
private void updateCOLData() {
  //update user defined class data
  userClass.doUpdate();
  
  // get data that should be removed from user defined class
  if (!userClass.RemoveFromCNCOneLayer().isEmpty()) {
    OneLayerDataToSend curDataOut;
    // create CNCOneLayerData from user defined class data that should be removed
    for(CNCOneLayerData curDataIn : userClass.RemoveFromCNCOneLayer()) {
      curDataOut = new OneLayerDataToSend(curDataIn.getPosition(), curDataIn.getMagnitude(), curDataIn.getDiameter(), 0, 0, true);
      // remove those data from CNCOneLayerData
      COLDataToSend.remove(curDataOut);
    }
  }

  // get data that should be added from user defined class
  if (!userClass.AddToCNCOneLayer().isEmpty()) {
    OneLayerDataToSend curDataOut;
    // create CNCOneLayerData from user defined class data that should be added
    for(CNCOneLayerData curDataIn : userClass.AddToCNCOneLayer()) {
      curDataOut = new OneLayerDataToSend(curDataIn.getPosition(), curDataIn.getMagnitude(), curDataIn.getDiameter(), 0, 0, true);
      curDataOut.setIsValid(isValidCNCOneLayer(curDataOut));
      // remove those data to CNCOneLayerData
      COLDataToSend.add(curDataOut); 
    }
  }
}

// this method update CMLData set
private void updateCMLData() {
}

// this method update ROLData set
private void updateROLData() {
}

// this method update RMLData set
private void updateRMLData() {
}

// user can send data to grasshopper by pressing s on keyboard
void keyPressed() {
  if (key == 's') {
    if (CNCOneLayerDisp.getState()) {
      readCNCOneLayerTextFields();
      CNCOneLayerSend.setState(true);
      CNCOneLayerSendEvent();
    }
    if (RBTOneLayerDisp.getState()) {
      readCNCMltLayerTextFields();
      CNCMltLayerSend.setState(true);
      CNCMltLayerSendEvent();
    }
    if (CNCMltLayerDisp.getState()) {
      readRBTOneLayerTextFields();
      RBTOneLayerSend.setState(true);
      RBTOneLayerSendEvent();
    }
    if (RBTMltLayerDisp.getState()) {
      readRBTMltLayerTextFields();
      RBTMltLayerSend.setState(true);
      RBTMltLayerSendEvent();
    }
  }
}
